# $Id: WhereNow.pm 7370 2012-04-09 01:17:33Z chris $

=head1 NAME

WebService::IMDB::WhereNow

=head1 DESCRIPTION

WebService::IMDB::WhereNow isa L<WebService::IMDB::DateText>.

=cut

package WebService::IMDB::WhereNow;

use strict;
use warnings;

our $VERSION = '0.05';

use base qw(WebService::IMDB::DateText);

1;
