# $Id: Birth.pm 7370 2012-04-09 01:17:33Z chris $

=head1 NAME

WebService::IMDB::Birth

=head1 DESCRIPTION

WebService::IMDB::Birth isa L<WebService::IMDB::DatePlace>.

=cut

package WebService::IMDB::Birth;

use strict;
use warnings;

our $VERSION = '0.05';

use base qw(WebService::IMDB::DatePlace);

1;
